import 'dart:async';
import 'dart:convert';
import "dart:math";

import 'package:flutter/services.dart';

import 'package:momomotus/utils/tools.dart';

class RandomPickWord {
  static Set<String> wordList = <String>{};
  static Set<String> dictionary = <String>{};

  static String _lang = '';
  static int _length = 5;
  static String _level = '';

  RandomPickWord();

  String _word = '';
  String get word => _word;

  init(String lang, int length, String level) async {
    if (lang != _lang ||
        length != _length ||
        level != _level ||
        wordList.isEmpty ||
        dictionary.isEmpty) {
      _lang = lang;
      _length = length;
      _level = level;

      dictionary.clear();
      wordList.clear();

      String wordBaseFilename = 'words-$length-$lang';

      // Get full dictionary (eligible words)
      printlog('Reload dictionary ($lang / $length)');
      try {
        String wordsFile = '$wordBaseFilename-dictionary';
        var data = await rootBundle.loadString('assets/files/$wordsFile.txt');
        LineSplitter.split(data).forEach((line) {
          if (line.length == length) {
            dictionary.add(line.toUpperCase());
          }
        });
      } catch (e) {
        throw "Failed loading dictionary database";
      }

      // Get guessable words list (will pick random word from)
      printlog('Reload words list ($lang / $length / $level)');
      try {
        String wordsFile = '$wordBaseFilename-$level';
        var data = await rootBundle.loadString('assets/files/$wordsFile.txt');
        LineSplitter.split(data).forEach((line) {
          if (line.length == length) {
            wordList.add(line.toUpperCase());
          }
        });
      } catch (e) {
        throw "Failed loading words database";
      }
    }

    printlog('Words in dictionary: ${dictionary.length}');
    printlog('Words in words list: ${wordList.length}');

    _word = '';
    await wordFromLocalFile();
  }

  Future<void> wordFromLocalFile() async {
    // Check we have enough words
    if (wordList.isEmpty) {
      printlog('Not enough words in list.');
      _word = '';
    } else {
      final random = Random();
      _word = wordList.elementAt(random.nextInt(wordList.length));
    }
  }

  static bool checkWordExists(String word) {
    return dictionary.contains(word);
  }
}
